import { Command as $Command } from "@smithy/smithy-client";
import { MetadataBearer as __MetadataBearer } from "@smithy/types";
import { CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes } from "../CloudWatchClient";
import { DeleteDashboardsInput, DeleteDashboardsOutput } from "../models/models_0";
/**
 * @public
 */
export type { __MetadataBearer };
export { $Command };
/**
 * @public
 *
 * The input for {@link DeleteDashboardsCommand}.
 */
export interface DeleteDashboardsCommandInput extends DeleteDashboardsInput {
}
/**
 * @public
 *
 * The output of {@link DeleteDashboardsCommand}.
 */
export interface DeleteDashboardsCommandOutput extends DeleteDashboardsOutput, __MetadataBearer {
}
declare const DeleteDashboardsCommand_base: {
    new (input: DeleteDashboardsCommandInput): import("@smithy/smithy-client").CommandImpl<DeleteDashboardsCommandInput, DeleteDashboardsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    new (input: DeleteDashboardsCommandInput): import("@smithy/smithy-client").CommandImpl<DeleteDashboardsCommandInput, DeleteDashboardsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    getEndpointParameterInstructions(): import("@smithy/middleware-endpoint").EndpointParameterInstructions;
};
/**
 * <p>Deletes all dashboards that you specify. You can specify up to 100 dashboards to
 *             delete. If there is an error during this call, no dashboards are deleted.</p>
 * @example
 * Use a bare-bones client and the command you need to make an API call.
 * ```javascript
 * import { CloudWatchClient, DeleteDashboardsCommand } from "@aws-sdk/client-cloudwatch"; // ES Modules import
 * // const { CloudWatchClient, DeleteDashboardsCommand } = require("@aws-sdk/client-cloudwatch"); // CommonJS import
 * const client = new CloudWatchClient(config);
 * const input = { // DeleteDashboardsInput
 *   DashboardNames: [ // DashboardNames // required
 *     "STRING_VALUE",
 *   ],
 * };
 * const command = new DeleteDashboardsCommand(input);
 * const response = await client.send(command);
 * // {};
 *
 * ```
 *
 * @param DeleteDashboardsCommandInput - {@link DeleteDashboardsCommandInput}
 * @returns {@link DeleteDashboardsCommandOutput}
 * @see {@link DeleteDashboardsCommandInput} for command's `input` shape.
 * @see {@link DeleteDashboardsCommandOutput} for command's `response` shape.
 * @see {@link CloudWatchClientResolvedConfig | config} for CloudWatchClient's `config` shape.
 *
 * @throws {@link DashboardNotFoundError} (client fault)
 *  <p>The specified dashboard does not exist.</p>
 *
 * @throws {@link InternalServiceFault} (server fault)
 *  <p>Request processing has failed due to some unknown error, exception, or
 *             failure.</p>
 *
 * @throws {@link InvalidParameterValueException} (client fault)
 *  <p>The value of an input parameter is bad or out-of-range.</p>
 *
 * @throws {@link CloudWatchServiceException}
 * <p>Base exception class for all service exceptions from CloudWatch service.</p>
 *
 *
 * @public
 */
export declare class DeleteDashboardsCommand extends DeleteDashboardsCommand_base {
    /** @internal type navigation helper, not in runtime. */
    protected static __types: {
        api: {
            input: DeleteDashboardsInput;
            output: {};
        };
        sdk: {
            input: DeleteDashboardsCommandInput;
            output: DeleteDashboardsCommandOutput;
        };
    };
}
