import { Command as $Command } from "@smithy/smithy-client";
import { MetadataBearer as __MetadataBearer } from "@smithy/types";
import { CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes } from "../CloudWatchClient";
import { DeleteMetricStreamInput, DeleteMetricStreamOutput } from "../models/models_0";
/**
 * @public
 */
export type { __MetadataBearer };
export { $Command };
/**
 * @public
 *
 * The input for {@link DeleteMetricStreamCommand}.
 */
export interface DeleteMetricStreamCommandInput extends DeleteMetricStreamInput {
}
/**
 * @public
 *
 * The output of {@link DeleteMetricStreamCommand}.
 */
export interface DeleteMetricStreamCommandOutput extends DeleteMetricStreamOutput, __MetadataBearer {
}
declare const DeleteMetricStreamCommand_base: {
    new (input: DeleteMetricStreamCommandInput): import("@smithy/smithy-client").CommandImpl<DeleteMetricStreamCommandInput, DeleteMetricStreamCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    new (input: DeleteMetricStreamCommandInput): import("@smithy/smithy-client").CommandImpl<DeleteMetricStreamCommandInput, DeleteMetricStreamCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    getEndpointParameterInstructions(): import("@smithy/middleware-endpoint").EndpointParameterInstructions;
};
/**
 * <p>Permanently deletes the metric stream that you specify.</p>
 * @example
 * Use a bare-bones client and the command you need to make an API call.
 * ```javascript
 * import { CloudWatchClient, DeleteMetricStreamCommand } from "@aws-sdk/client-cloudwatch"; // ES Modules import
 * // const { CloudWatchClient, DeleteMetricStreamCommand } = require("@aws-sdk/client-cloudwatch"); // CommonJS import
 * const client = new CloudWatchClient(config);
 * const input = { // DeleteMetricStreamInput
 *   Name: "STRING_VALUE", // required
 * };
 * const command = new DeleteMetricStreamCommand(input);
 * const response = await client.send(command);
 * // {};
 *
 * ```
 *
 * @param DeleteMetricStreamCommandInput - {@link DeleteMetricStreamCommandInput}
 * @returns {@link DeleteMetricStreamCommandOutput}
 * @see {@link DeleteMetricStreamCommandInput} for command's `input` shape.
 * @see {@link DeleteMetricStreamCommandOutput} for command's `response` shape.
 * @see {@link CloudWatchClientResolvedConfig | config} for CloudWatchClient's `config` shape.
 *
 * @throws {@link InternalServiceFault} (server fault)
 *  <p>Request processing has failed due to some unknown error, exception, or
 *             failure.</p>
 *
 * @throws {@link InvalidParameterValueException} (client fault)
 *  <p>The value of an input parameter is bad or out-of-range.</p>
 *
 * @throws {@link MissingRequiredParameterException} (client fault)
 *  <p>An input parameter that is required is missing.</p>
 *
 * @throws {@link CloudWatchServiceException}
 * <p>Base exception class for all service exceptions from CloudWatch service.</p>
 *
 *
 * @public
 */
export declare class DeleteMetricStreamCommand extends DeleteMetricStreamCommand_base {
    /** @internal type navigation helper, not in runtime. */
    protected static __types: {
        api: {
            input: DeleteMetricStreamInput;
            output: {};
        };
        sdk: {
            input: DeleteMetricStreamCommandInput;
            output: DeleteMetricStreamCommandOutput;
        };
    };
}
