import { Command as $Command } from "@smithy/smithy-client";
import { MetadataBearer as __MetadataBearer } from "@smithy/types";
import { CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes } from "../CloudWatchClient";
import { DisableAlarmActionsInput } from "../models/models_0";
/**
 * @public
 */
export type { __MetadataBearer };
export { $Command };
/**
 * @public
 *
 * The input for {@link DisableAlarmActionsCommand}.
 */
export interface DisableAlarmActionsCommandInput extends DisableAlarmActionsInput {
}
/**
 * @public
 *
 * The output of {@link DisableAlarmActionsCommand}.
 */
export interface DisableAlarmActionsCommandOutput extends __MetadataBearer {
}
declare const DisableAlarmActionsCommand_base: {
    new (input: DisableAlarmActionsCommandInput): import("@smithy/smithy-client").CommandImpl<DisableAlarmActionsCommandInput, DisableAlarmActionsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    new (input: DisableAlarmActionsCommandInput): import("@smithy/smithy-client").CommandImpl<DisableAlarmActionsCommandInput, DisableAlarmActionsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    getEndpointParameterInstructions(): import("@smithy/middleware-endpoint").EndpointParameterInstructions;
};
/**
 * <p>Disables the actions for the specified alarms. When an alarm's actions are
 *             disabled, the alarm actions do not execute when the alarm state changes.</p>
 * @example
 * Use a bare-bones client and the command you need to make an API call.
 * ```javascript
 * import { CloudWatchClient, DisableAlarmActionsCommand } from "@aws-sdk/client-cloudwatch"; // ES Modules import
 * // const { CloudWatchClient, DisableAlarmActionsCommand } = require("@aws-sdk/client-cloudwatch"); // CommonJS import
 * const client = new CloudWatchClient(config);
 * const input = { // DisableAlarmActionsInput
 *   AlarmNames: [ // AlarmNames // required
 *     "STRING_VALUE",
 *   ],
 * };
 * const command = new DisableAlarmActionsCommand(input);
 * const response = await client.send(command);
 * // {};
 *
 * ```
 *
 * @param DisableAlarmActionsCommandInput - {@link DisableAlarmActionsCommandInput}
 * @returns {@link DisableAlarmActionsCommandOutput}
 * @see {@link DisableAlarmActionsCommandInput} for command's `input` shape.
 * @see {@link DisableAlarmActionsCommandOutput} for command's `response` shape.
 * @see {@link CloudWatchClientResolvedConfig | config} for CloudWatchClient's `config` shape.
 *
 * @throws {@link CloudWatchServiceException}
 * <p>Base exception class for all service exceptions from CloudWatch service.</p>
 *
 *
 * @public
 */
export declare class DisableAlarmActionsCommand extends DisableAlarmActionsCommand_base {
    /** @internal type navigation helper, not in runtime. */
    protected static __types: {
        api: {
            input: DisableAlarmActionsInput;
            output: {};
        };
        sdk: {
            input: DisableAlarmActionsCommandInput;
            output: DisableAlarmActionsCommandOutput;
        };
    };
}
