import { Command as $Command } from "@smithy/smithy-client";
import { MetadataBearer as __MetadataBearer } from "@smithy/types";
import { CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes } from "../CloudWatchClient";
import { ListMetricStreamsInput, ListMetricStreamsOutput } from "../models/models_0";
/**
 * @public
 */
export type { __MetadataBearer };
export { $Command };
/**
 * @public
 *
 * The input for {@link ListMetricStreamsCommand}.
 */
export interface ListMetricStreamsCommandInput extends ListMetricStreamsInput {
}
/**
 * @public
 *
 * The output of {@link ListMetricStreamsCommand}.
 */
export interface ListMetricStreamsCommandOutput extends ListMetricStreamsOutput, __MetadataBearer {
}
declare const ListMetricStreamsCommand_base: {
    new (input: ListMetricStreamsCommandInput): import("@smithy/smithy-client").CommandImpl<ListMetricStreamsCommandInput, ListMetricStreamsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    new (...[input]: [] | [ListMetricStreamsCommandInput]): import("@smithy/smithy-client").CommandImpl<ListMetricStreamsCommandInput, ListMetricStreamsCommandOutput, CloudWatchClientResolvedConfig, ServiceInputTypes, ServiceOutputTypes>;
    getEndpointParameterInstructions(): import("@smithy/middleware-endpoint").EndpointParameterInstructions;
};
/**
 * <p>Returns a list of metric streams in this account.</p>
 * @example
 * Use a bare-bones client and the command you need to make an API call.
 * ```javascript
 * import { CloudWatchClient, ListMetricStreamsCommand } from "@aws-sdk/client-cloudwatch"; // ES Modules import
 * // const { CloudWatchClient, ListMetricStreamsCommand } = require("@aws-sdk/client-cloudwatch"); // CommonJS import
 * const client = new CloudWatchClient(config);
 * const input = { // ListMetricStreamsInput
 *   NextToken: "STRING_VALUE",
 *   MaxResults: Number("int"),
 * };
 * const command = new ListMetricStreamsCommand(input);
 * const response = await client.send(command);
 * // { // ListMetricStreamsOutput
 * //   NextToken: "STRING_VALUE",
 * //   Entries: [ // MetricStreamEntries
 * //     { // MetricStreamEntry
 * //       Arn: "STRING_VALUE",
 * //       CreationDate: new Date("TIMESTAMP"),
 * //       LastUpdateDate: new Date("TIMESTAMP"),
 * //       Name: "STRING_VALUE",
 * //       FirehoseArn: "STRING_VALUE",
 * //       State: "STRING_VALUE",
 * //       OutputFormat: "json" || "opentelemetry0.7" || "opentelemetry1.0",
 * //     },
 * //   ],
 * // };
 *
 * ```
 *
 * @param ListMetricStreamsCommandInput - {@link ListMetricStreamsCommandInput}
 * @returns {@link ListMetricStreamsCommandOutput}
 * @see {@link ListMetricStreamsCommandInput} for command's `input` shape.
 * @see {@link ListMetricStreamsCommandOutput} for command's `response` shape.
 * @see {@link CloudWatchClientResolvedConfig | config} for CloudWatchClient's `config` shape.
 *
 * @throws {@link InternalServiceFault} (server fault)
 *  <p>Request processing has failed due to some unknown error, exception, or
 *             failure.</p>
 *
 * @throws {@link InvalidNextToken} (client fault)
 *  <p>The next token specified is invalid.</p>
 *
 * @throws {@link InvalidParameterValueException} (client fault)
 *  <p>The value of an input parameter is bad or out-of-range.</p>
 *
 * @throws {@link MissingRequiredParameterException} (client fault)
 *  <p>An input parameter that is required is missing.</p>
 *
 * @throws {@link CloudWatchServiceException}
 * <p>Base exception class for all service exceptions from CloudWatch service.</p>
 *
 *
 * @public
 */
export declare class ListMetricStreamsCommand extends ListMetricStreamsCommand_base {
    /** @internal type navigation helper, not in runtime. */
    protected static __types: {
        api: {
            input: ListMetricStreamsInput;
            output: ListMetricStreamsOutput;
        };
        sdk: {
            input: ListMetricStreamsCommandInput;
            output: ListMetricStreamsCommandOutput;
        };
    };
}
