/*
 * Copyright (c) Microsoft Corporation.
 * Licensed under the MIT License.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */
import { __awaiter } from "tslib";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
import { ContainerInstanceManagementClient } from "@azure/arm-containerinstance";
import { DefaultAzureCredential } from "@azure/identity";
import * as dotenv from "dotenv";
dotenv.config();
/**
 * This sample demonstrates how to Create or update container groups with specified configurations.
 *
 * @summary Create or update container groups with specified configurations.
 * x-ms-original-file: specification/containerinstance/resource-manager/Microsoft.ContainerInstance/stable/2023-05-01/examples/ContainerGroupCreateConfidential.json
 */
function confidentialContainerGroup() {
    return __awaiter(this, void 0, void 0, function* () {
        const subscriptionId = process.env["CONTAINERINSTANCE_SUBSCRIPTION_ID"] || "subid";
        const resourceGroupName = process.env["CONTAINERINSTANCE_RESOURCE_GROUP"] || "demo";
        const containerGroupName = "demo1";
        const containerGroup = {
            confidentialComputeProperties: {
                ccePolicy: "eyJhbGxvd19hbGwiOiB0cnVlLCAiY29udGFpbmVycyI6IHsibGVuZ3RoIjogMCwgImVsZW1lbnRzIjogbnVsbH19"
            },
            containers: [
                {
                    name: "accdemo",
                    command: [],
                    environmentVariables: [],
                    image: "confiimage",
                    ports: [{ port: 8000 }],
                    resources: { requests: { cpu: 1, memoryInGB: 1.5 } },
                    securityContext: {
                        capabilities: { add: ["CAP_NET_ADMIN"] },
                        privileged: false
                    }
                }
            ],
            imageRegistryCredentials: [],
            ipAddress: { type: "Public", ports: [{ port: 8000, protocol: "TCP" }] },
            location: "westeurope",
            osType: "Linux",
            sku: "Confidential"
        };
        const credential = new DefaultAzureCredential();
        const client = new ContainerInstanceManagementClient(credential, subscriptionId);
        const result = yield client.containerGroups.beginCreateOrUpdateAndWait(resourceGroupName, containerGroupName, containerGroup);
        console.log(result);
    });
}
/**
 * This sample demonstrates how to Create or update container groups with specified configurations.
 *
 * @summary Create or update container groups with specified configurations.
 * x-ms-original-file: specification/containerinstance/resource-manager/Microsoft.ContainerInstance/stable/2023-05-01/examples/ContainerGroupExtensions.json
 */
function containerGroupCreateWithExtensions() {
    return __awaiter(this, void 0, void 0, function* () {
        const subscriptionId = process.env["CONTAINERINSTANCE_SUBSCRIPTION_ID"] || "subid";
        const resourceGroupName = process.env["CONTAINERINSTANCE_RESOURCE_GROUP"] || "demo";
        const containerGroupName = "demo1";
        const containerGroup = {
            containers: [
                {
                    name: "demo1",
                    command: [],
                    environmentVariables: [],
                    image: "nginx",
                    ports: [{ port: 80 }],
                    resources: { requests: { cpu: 1, memoryInGB: 1.5 } }
                }
            ],
            imageRegistryCredentials: [],
            ipAddress: { type: "Private", ports: [{ port: 80, protocol: "TCP" }] },
            location: "eastus2",
            osType: "Linux",
            subnetIds: [
                {
                    id: "/subscriptions/00000000-0000-0000-0000-00000000/resourceGroups/test-rg/providers/Microsoft.Network/virtualNetworks/test-rg-vnet/subnets/test-subnet"
                }
            ],
            extensions: [
                {
                    name: "kube-proxy",
                    extensionType: "kube-proxy",
                    protectedSettings: { kubeConfig: "<kubeconfig encoded string>" },
                    settings: { clusterCidr: "10.240.0.0/16", kubeVersion: "v1.9.10" },
                    version: "1.0"
                },
                {
                    name: "vk-realtime-metrics",
                    extensionType: "realtime-metrics",
                    version: "1.0"
                }
            ]
        };
        const credential = new DefaultAzureCredential();
        const client = new ContainerInstanceManagementClient(credential, subscriptionId);
        const result = yield client.containerGroups.beginCreateOrUpdateAndWait(resourceGroupName, containerGroupName, containerGroup);
        console.log(result);
    });
}
/**
 * This sample demonstrates how to Create or update container groups with specified configurations.
 *
 * @summary Create or update container groups with specified configurations.
 * x-ms-original-file: specification/containerinstance/resource-manager/Microsoft.ContainerInstance/stable/2023-05-01/examples/ContainerGroupEncryptionProperties.json
 */
function containerGroupWithEncryptionProperties() {
    return __awaiter(this, void 0, void 0, function* () {
        const subscriptionId = process.env["CONTAINERINSTANCE_SUBSCRIPTION_ID"] || "subid";
        const resourceGroupName = process.env["CONTAINERINSTANCE_RESOURCE_GROUP"] || "demo";
        const containerGroupName = "demo1";
        const containerGroup = {
            containers: [
                {
                    name: "demo1",
                    command: [],
                    environmentVariables: [],
                    image: "nginx",
                    ports: [{ port: 80 }],
                    resources: { requests: { cpu: 1, memoryInGB: 1.5 } }
                }
            ],
            encryptionProperties: {
                identity: "/subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/test-rg/providers/Microsoft.ManagedIdentity/userAssignedIdentities/container-group-identity",
                keyName: "test-key",
                keyVersion: "<key version>",
                vaultBaseUrl: "https://testkeyvault.vault.azure.net"
            },
            identity: {
                type: "UserAssigned",
                userAssignedIdentities: {
                    "/subscriptions/00000000000000000000000000000000/resourcegroups/testRg/providers/MicrosoftManagedIdentity/userAssignedIdentities/containerGroupIdentity": {}
                }
            },
            imageRegistryCredentials: [],
            ipAddress: { type: "Public", ports: [{ port: 80, protocol: "TCP" }] },
            location: "eastus2",
            osType: "Linux"
        };
        const credential = new DefaultAzureCredential();
        const client = new ContainerInstanceManagementClient(credential, subscriptionId);
        const result = yield client.containerGroups.beginCreateOrUpdateAndWait(resourceGroupName, containerGroupName, containerGroup);
        console.log(result);
    });
}
/**
 * This sample demonstrates how to Create or update container groups with specified configurations.
 *
 * @summary Create or update container groups with specified configurations.
 * x-ms-original-file: specification/containerinstance/resource-manager/Microsoft.ContainerInstance/stable/2023-05-01/examples/ContainerGroupsCreateOrUpdate.json
 */
function containerGroupsCreateOrUpdate() {
    return __awaiter(this, void 0, void 0, function* () {
        const subscriptionId = process.env["CONTAINERINSTANCE_SUBSCRIPTION_ID"] || "subid";
        const resourceGroupName = process.env["CONTAINERINSTANCE_RESOURCE_GROUP"] || "demo";
        const containerGroupName = "demo1";
        const containerGroup = {
            containers: [
                {
                    name: "demo1",
                    command: [],
                    environmentVariables: [],
                    image: "nginx",
                    ports: [{ port: 80 }],
                    resources: {
                        requests: { cpu: 1, gpu: { count: 1, sku: "K80" }, memoryInGB: 1.5 }
                    },
                    volumeMounts: [
                        { name: "volume1", mountPath: "/mnt/volume1", readOnly: false },
                        { name: "volume2", mountPath: "/mnt/volume2", readOnly: false },
                        { name: "volume3", mountPath: "/mnt/volume3", readOnly: true }
                    ]
                }
            ],
            diagnostics: {
                logAnalytics: {
                    logType: "ContainerInsights",
                    metadata: { testKey: "test-metadata-value" },
                    workspaceId: "workspaceid",
                    workspaceKey: "workspaceKey",
                    workspaceResourceId: "/subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/rg/providers/microsoft.operationalinsights/workspaces/workspace"
                }
            },
            dnsConfig: {
                nameServers: ["1.1.1.1"],
                options: "ndots:2",
                searchDomains: "cluster.local svc.cluster.local"
            },
            identity: {
                type: "SystemAssigned, UserAssigned",
                userAssignedIdentities: {
                    "/subscriptions/00000000000000000000000000000000/resourceGroups/myResourceGroup/providers/MicrosoftManagedIdentity/userAssignedIdentities/identityName": {}
                }
            },
            imageRegistryCredentials: [],
            ipAddress: {
                type: "Public",
                autoGeneratedDomainNameLabelScope: "Unsecure",
                dnsNameLabel: "dnsnamelabel1",
                ports: [{ port: 80, protocol: "TCP" }]
            },
            location: "west us",
            osType: "Linux",
            subnetIds: [
                {
                    id: "[resourceId('Microsoft.Network/virtualNetworks/subnets', parameters('vnetName'), parameters('subnetName'))]"
                }
            ],
            volumes: [
                {
                    name: "volume1",
                    azureFile: {
                        shareName: "shareName",
                        storageAccountKey: "accountKey",
                        storageAccountName: "accountName"
                    }
                },
                { name: "volume2", emptyDir: {} },
                {
                    name: "volume3",
                    secret: {
                        secretKey1: "SecretValue1InBase64",
                        secretKey2: "SecretValue2InBase64"
                    }
                }
            ]
        };
        const credential = new DefaultAzureCredential();
        const client = new ContainerInstanceManagementClient(credential, subscriptionId);
        const result = yield client.containerGroups.beginCreateOrUpdateAndWait(resourceGroupName, containerGroupName, containerGroup);
        console.log(result);
    });
}
/**
 * This sample demonstrates how to Create or update container groups with specified configurations.
 *
 * @summary Create or update container groups with specified configurations.
 * x-ms-original-file: specification/containerinstance/resource-manager/Microsoft.ContainerInstance/stable/2023-05-01/examples/ContainerGroupsCreatePriority.json
 */
function containerGroupsCreateWithPriority() {
    return __awaiter(this, void 0, void 0, function* () {
        const subscriptionId = process.env["CONTAINERINSTANCE_SUBSCRIPTION_ID"] || "subid";
        const resourceGroupName = process.env["CONTAINERINSTANCE_RESOURCE_GROUP"] || "demo";
        const containerGroupName = "demo1";
        const containerGroup = {
            containers: [
                {
                    name: "test-container-001",
                    command: ["/bin/sh", "-c", "sleep 10"],
                    image: "alpine:latest",
                    resources: { requests: { cpu: 1, memoryInGB: 1 } }
                }
            ],
            location: "eastus",
            osType: "Linux",
            priority: "Spot",
            restartPolicy: "Never",
            sku: "Standard"
        };
        const credential = new DefaultAzureCredential();
        const client = new ContainerInstanceManagementClient(credential, subscriptionId);
        const result = yield client.containerGroups.beginCreateOrUpdateAndWait(resourceGroupName, containerGroupName, containerGroup);
        console.log(result);
    });
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        confidentialContainerGroup();
        containerGroupCreateWithExtensions();
        containerGroupWithEncryptionProperties();
        containerGroupsCreateOrUpdate();
        containerGroupsCreateWithPriority();
    });
}
main().catch(console.error);
//# sourceMappingURL=containerGroupsCreateOrUpdateSample.js.map