"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mungeMocks = void 0;
const module_1 = require("module");
const path_1 = require("path");
const is_relative_require_1 = require("resolve-import/is-relative-require");
const url_1 = require("url");
// turn all the mocks we get into either the exports provided,
// if it's an object, or { default: <value> } otherwise.
const mungeMocks = (mocksInput, dir) => {
    const mocks = Object.create(null);
    for (const [k, v] of Object.entries(mocksInput)) {
        const m = v && typeof v === 'object' && !Array.isArray(v) ?
            v
            : { default: v };
        if ((0, module_1.isBuiltin)(k)) {
            mocks[k] = m;
            /* c8 ignore start */
            if (k.startsWith('node:')) {
                /* c8 ignore stop */
                const bare = k.substring('node:'.length);
                if (!(bare in mocksInput)) {
                    mocks[bare] = m;
                }
            }
            else {
                const prefixed = `node:${k}`;
                if (!(prefixed in mocksInput)) {
                    mocks[prefixed] = m;
                }
            }
        }
        else if (/^(file:\/\/\/|https?:\/\/)/.test(k)) {
            mocks[k] = m;
        }
        else if ((0, is_relative_require_1.isRelativeRequire)(k) || (0, path_1.isAbsolute)(k)) {
            mocks[String((0, url_1.pathToFileURL)((0, path_1.resolve)(dir, k)))] = m;
        }
        else {
            // absolute package name
            mocks[k] = m;
        }
    }
    return mocks;
};
exports.mungeMocks = mungeMocks;
//# sourceMappingURL=munge-mocks.js.map