"use strict";
// The sources reported in sourcemaps can be fickle,
// and istanbul's reporting doesn't handle it so well.
// So, we canonicalize all entries in sources to a file://
// url with consistent capitalization.
Object.defineProperty(exports, "__esModule", { value: true });
exports.canonicalPath = exports.canonicalSource = void 0;
const path_1 = require("path");
const url_1 = require("url");
const cwd = process.cwd();
const canonicalSource = (s) => {
    // convert to a path
    if (s.startsWith('file://')) {
        const u = new URL(s);
        const hash = u.hash;
        const search = u.search;
        return String((0, url_1.pathToFileURL)((0, exports.canonicalPath)((0, url_1.fileURLToPath)(s)))) +
            hash + search;
    }
    // re-resolve it relative to the cwd
    // back to a file: url
    return String((0, url_1.pathToFileURL)((0, exports.canonicalPath)(s)));
};
exports.canonicalSource = canonicalSource;
const canonicalPath = (s) => {
    const rel = (0, path_1.relative)(cwd, s);
    s = (0, path_1.resolve)(cwd, rel);
    // capitalize drive letters
    if (/^[a-z]:/.test(s))
        s = s.charAt(0).toUpperCase() + s.substring(1);
    // capitalize UNC paths
    if (s.startsWith('\\\\')) {
        s = s.replace(/\\\\[^\\]+\\[^\\]+\\/, _0 => _0.toUpperCase());
    }
    return s;
};
exports.canonicalPath = canonicalPath;
//# sourceMappingURL=canonical-source.js.map