'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _each2 = require('../_each');

var _each3 = _interopRequireDefault(_each2);

var _span_context_imp = require('./span_context_imp');

var _span_context_imp2 = _interopRequireDefault(_span_context_imp);

var _propagator_ls = require('./propagator_ls');

var _propagator_ls2 = _interopRequireDefault(_propagator_ls);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CARRIER_DD_TRACER_STATE_PREFIX = 'x-datadog-';

var DDPropagator = function () {
    function DDPropagator(tracer) {
        _classCallCheck(this, DDPropagator);

        this._tracer = tracer;
        this._baggagePrefix = _propagator_ls2.default;
        this._carrierPrefix = CARRIER_DD_TRACER_STATE_PREFIX;
    }

    _createClass(DDPropagator, [{
        key: 'inject',
        value: function inject(spanContext, carrier) {
            var _this = this;

            if (!carrier) {
                this._tracer._error('Unexpected null carrier in call to inject');
                return;
            }
            if ((typeof carrier === 'undefined' ? 'undefined' : _typeof(carrier)) !== 'object') {
                this._tracer._error('Unexpected \'' + (typeof carrier === 'undefined' ? 'undefined' : _typeof(carrier)) + '\' FORMAT_TEXT_MAP carrier in call to inject');
                return;
            }

            // eslint-disable-next-line max-len
            // Per https://github.com/lightstep/lightstep-tracer-javascript/blob/master/src/imp/platform/node/platform_node.js#L91
            // all generated GUIDs are base 16 strings.
            // DD headers expect integers, not base 16 values.
            carrier[this._carrierPrefix + 'parent-id'] = parseInt(spanContext._guid, 16).toString();
            carrier[this._carrierPrefix + 'trace-id'] = parseInt(spanContext.traceGUID(), 16).toString();
            if (spanContext._sampled) {
                carrier[this._carrierPrefix + 'sampling-priority'] = '1';
            } else {
                carrier[this._carrierPrefix + 'sampling-priority'] = '0';
            }

            spanContext.forEachBaggageItem(function (key, value) {
                carrier['' + _this._baggagePrefix + key] = value;
            });
            return carrier;
        }
    }, {
        key: 'extract',
        value: function extract(carrier) {
            var _this2 = this;

            // Iterate over the contents of the carrier and set the properties
            // accordingly.
            var foundFields = 0;
            var spanGUID = null;
            var traceGUID = null;
            var sampled = true;

            (0, _each3.default)(carrier, function (value, key) {
                key = key.toLowerCase();
                if (key.substr(0, _this2._carrierPrefix.length) !== _this2._carrierPrefix) {
                    return;
                }
                var suffix = key.substr(_this2._carrierPrefix.length);

                switch (suffix) {
                    case 'trace-id':
                        foundFields++;
                        traceGUID = parseInt(value, 10).toString(16);
                        break;
                    case 'parent-id':
                        foundFields++;
                        spanGUID = parseInt(value, 10).toString(16);
                        break;
                    case 'sampling-priority':
                        // TODO: support sampling priority somehow. This is a float64 that does not
                        // necessarily represent the sampling decision
                        if (value === 0) {
                            sampled = false;
                        }
                        break;
                    default:
                        _this2._tracer._error('Unrecognized carrier key \'' + key + '\' with recognized prefix. Ignoring.');
                        break;
                }
            });

            if (foundFields === 0) {
                // This is not an error per se, there was simply no SpanContext
                // in the carrier.
                return null;
            }
            if (foundFields < 2) {
                // A partial SpanContext suggests some sort of data corruption.
                this._tracer._error('Only found a partial SpanContext: ' + carrier);
                return null;
            }

            var spanContext = new _span_context_imp2.default(spanGUID, traceGUID, sampled);

            (0, _each3.default)(carrier, function (value, key) {
                key = key.toLowerCase();
                if (key.substr(0, _this2._baggagePrefix.length) !== _this2._baggagePrefix) {
                    return;
                }
                var suffix = key.substr(_this2._baggagePrefix.length);
                spanContext.setBaggageItem(suffix, value);
            });
            return spanContext;
        }
    }]);

    return DDPropagator;
}();

exports.default = DDPropagator;
module.exports = exports.default;

//# sourceMappingURL=propagator_dd.js.map